#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

/* enable/disable a specific DMA channel */
extern void enable_dma(unsigned int dmanr);

#include <linux/config.h>

static __inline__ void disable_dma(unsigned int dmanr)
{
    switch(dmanr) {
#ifdef CONFIG_BLK_DEV_FD
    /* This is a HACK - at the moment ARM Linux doesn't support two FIQs
     being serviced together, until it does this hack must stay - DAG */
	case 0: { /* Data DMA */
    /* We have to replace it by FIQ code for command end */
    unsigned long flags;
    extern void fdc1772_comendhandler(void);
    /*printk("disable_dma for FD1772\n");*/
    disable_irq(16);

    save_flags(flags);
    cliIF();
    *((unsigned int *)0x1c)=0xea000000 | (((unsigned int)fdc1772_comendhandler-(0x1c+8))/4); /* B fdc1772_comendhandler */ 
    restore_flags(flags);
  }
  break;

#endif
	default: printk("disable_dma: dma %d not supported\n", dmanr); break;
    }
}

/* Clear the 'DMA Pointer Flip Flop'.
 * Write 0 for LSB/MSB, 1 for MSB/LSB access.
 * Use this once to initialize the FF to a known state.
 * After that, keep track of it. :-)
 * --- In order to do that, the DMA routines below should ---
 * --- only be used while interrupts are disabled! ---
 */
static __inline__ void clear_dma_ff(unsigned int dmanr)
{
    switch(dmanr) {
	case 2:  break;
	default: printk("clear_dma_ff: dma %d not supported\n", dmanr); break;
    }
}

/* set mode (above) for a specific DMA channel */
extern void set_dma_mode(unsigned int dmanr, char mode);

/* Set only the page register bits of the transfer address.
 * This is used for successive transfers when we know the contents of
 * the lower 16 bits of the DMA current address register, but a 64k boundary
 * may have been crossed.
 */
static __inline__ void set_dma_page(unsigned int dmanr, char pagenr)
{
    printk("set_dma_page: dma %d not supported\n", dmanr);
}


/* Set transfer address & page bits for specific DMA channel.
 * Assumes dma flipflop is clear.
 */
extern void set_dma_addr(unsigned int dmanr, unsigned int addr);

/* Set transfer size for a specific DMA channel.
 */
extern void set_dma_count(unsigned int dmanr, unsigned int count);

/* Get DMA residue count. After a DMA transfer, this
 * should return zero. Reading this while a DMA transfer is
 * still in progress will return unpredictable results.
 * If called before the channel has been used, it may return 1.
 * Otherwise, it returns the number of _bytes_ left to transfer.
 *
 * Assumes DMA flip-flop is clear.
 */
extern int get_dma_residue(unsigned int dmanr);

#endif /* _ASM_ARCH_DMA_H */

