/*
 * linux/include/asm-arm/ecard.h
 *
 * definitions for expansion cards
 *
 * This is a new system as from Linux 1.2.3
 *
 * Reference: Acorns Risc OS 3 Programmers Reference Manuals.
 *
 * Do not use the r_ fields - use the #defined ones
 */

/*
 * Expansion card header
 */

struct ex_ecld
{
	unsigned char  r_ecld;
	unsigned char  r_reserved[2];
	unsigned char  r_product[2];
	unsigned char  r_manufacturer[2];
	unsigned char  r_country;
	         long  r_fiqs;
	         long  r_irqs;
#define e_ecld(x)	((x)->r_ecld)
#define e_cd(x)		((x)->r_reserved[0] & 1)
#define e_is(x)		((x)->r_reserved[0] & 2)
#define e_w(x)		(((x)->r_reserved[0] & 12)>>2)
#define e_prod(x)	((x)->r_product[0]|((x)->r_product[1]<<8))
#define e_manu(x)	((x)->r_manufacturer[0]|((x)->r_manufacturer[1]<<8))
#define e_country(x)	((x)->r_country)
#define e_fiqmask(x)	((x)->r_fiqs & 0xff)
#define e_fiqaddr(x)	((x)->r_fiqs >> 8)
#define e_irqmask(x)	((x)->r_irqs & 0xff)
#define e_irqaddr(x)	((x)->r_irqs >> 8)
};

/*
 * Chunk directory entry
 */

struct chunk_dir
{
	unsigned char r_id;
	unsigned char r_len[3];
	unsigned long r_start;
	union
	{
		char string[256];
		char data[1];
	} d;
#define c_id(x)		((x)->r_id)
#define c_len(x)	((x)->r_len[0]|((x)->r_len[1]<<8)|((x)->r_len[2]<<16))
#define c_start(x)	((x)->r_start)
};

/*
 * This contains all the info needed on an expansion card
 */

struct expansion_card
{
	struct ex_ecld ecld;
	unsigned char irq;
	unsigned char fiq;
	unsigned char slot_no;
	unsigned char unused2;
	unsigned char *r_loader; /* loader program */
	volatile unsigned char *r_podaddr;
	volatile unsigned char *r_irqaddr;
	volatile unsigned char *r_fiqaddr;
	unsigned char r_irqmask;
	unsigned char r_fiqmask;
	unsigned char r_found;
	unsigned char r_claimed;
};

/*
 * functions exported
 */

#define ecard_claim(ec) ((ec)->r_claimed = 1)

#define ecard_release(ec) ((ec)->r_claimed = 0)
 
/*
 * Find an expansion card with the correct cld, product and manufacturer code
 */
extern struct expansion_card *ecard_find(int cld, int num, const int prod[], const int manu[]);
 
/*
 * Read a chunk from an expansion card
 * cd : where to put read data
 * ec : expansion card info struct
 * id : id number to find
 * num: (n+1)'th id to find.
 */
extern int ecard_readchunk(struct chunk_dir *cd, struct expansion_card *ec, int id, int num);

#define ECARD_IOC	0
#define ECARD_MEMC	1
/*
 * Obtain the address of a card
 */
#define ECARD_SLOW	0
#define ECARD_MEDIUM	1
#define ECARD_FAST	2
#define ECARD_SYNC	3
extern unsigned char *ecard_address (int card, int memc, int speed);
