#ifndef __ASM_ARM_IO_H
#define __ASM_ARM_IO_H

/*
 * Change virtual address to physical addresses
 */
extern inline unsigned long virt_to_phys(volatile void * address)
{
	return (unsigned long) address;
}

extern inline void * phys_to_virt(unsigned long address)
{
	return (void *) address;
}

extern unsigned long io_base;

/*
 * IO -> physical
 */
#define virt_to_bus virt_to_phys
#define bus_to_virt phys_to_virt

#undef USE_INLINE

#ifdef USE_INLINE
extern inline void __outb(unsigned char value, unsigned int port)		\
{										\
	__asm__ __volatile__(							\
	"cmp\t%2, #0x00c00000\n\t"						\
	"movge\t%0, #0\n\t"							\
	"movlt\t%0, #0x03000000\n\t"						\
	"orrlt\t%0, %0, #0x00010000\n\t"					\
	"strb\t%1, [%0, %2, lsl #2\n\t"	:					\
	"=r" (unused) : "r" (ret), "r" (port));				\
}

extern inline void __inb(unsigned int port)					\
{										\
	unsigned char ret, unused;						\
	__asm__ __volatile__(							\
	"cmp\t%1, #0x00c00000\n\t"						\
	"movge\t%2, #0\n\t"							\
	"movlt\t%2, #0x03000000\n\t"						\
	"orrlt\t%2, %2, #0x00010000\n\t"					\
	"ldrb\t%0, [%2, %1, lsl #2\n\t"	:					\
	"=r" (ret), "=r" (unused) : "r" (port));				\
	return ret;								\
}
	
#define __OUT1(s, x) \
extern inline void __out##s(unsigned x value, unsigned int port)		\
{										\
	if(port < 0x00C00000)							\
		*(unsigned x *)(0x03010000+(port << 2)) = value;		\
	else									\
		*(unsigned x *)(port << 2) = value;				\
}

#define __IN1(s, x) \
extern inline unsigned int __in##s(unsigned int port)				\
{										\
	if(port < 0x00C00000)							\
		return *(unsigned x *)(0x03010000+(port << 2));			\
	else									\
		return *(unsigned x *)(port << 2);				\
}
#else

#define __OUT1(s, x) \
extern void __out##s(unsigned x value, unsigned int port);
#define __IN1(s, x) \
extern unsigned x __in##s(unsigned int port);
#endif

#define __OUT(s, x)  \
  __OUT1(s     , x); \
  __OUT1(s##c  , x); \
  __OUT1(s##_p , x); \
  __OUT1(s##c_p, x);

__OUT(b, char);
__OUT(w, short);
__OUT(l, long);

#define __IN(s, x)  \
  __IN1(s     , x); \
  __IN1(s##c  , x); \
  __IN1(s##_p , x); \
  __IN1(s##c_p, x);

__IN(b, char);
__IN(w, short);
__IN(l, long);

#ifdef USE_INLINE
#define __INS1(s, x) \
extern inline void ins##s(unsigned int port, void *to, int len)			\
{										\
	unsigned x *p = (unsigned x *)to;					\
	if(port < 0x00C00000)							\
		port = 0x03010000+(port << 2);					\
	else									\
		port <<= 2;							\
	while(len-- > 0)							\
		*p++ = *(unsigned x *)port;					\
}

#define __OUTS1(s, x) \
extern inline void outs##s(unsigned int port, const void *from, int len)	\
{										\
	unsigned x *p = (unsigned x *)from;					\
	if(port < 0x00C00000)							\
		port = 0x03010000+(port << 2);					\
	else									\
		port <<= 2;							\
	while(len-- > 0)							\
		*(unsigned x *)port = *p++;					\
}

#else

#define __INS1(s, x) \
extern void ins##s(unsigned int port, void *to, int len);

#define __OUTS1(s, x) \
extern void outs##s(unsigned int port, const void *from, int len);

#endif

#define __INS(s, x)	\
  __INS1(s     , x);	\
  __INS1(s##c  , x);	\
  __INS1(s##_p , x);	\
  __INS1(s##c_p, x);

__INS(b, char);
__INS(w, short);
__INS(l, long);

#define __OUTS(s, x)	\
  __OUTS1(s     , x);	\
  __OUTS1(s##c  , x);	\
  __OUTS1(s##_p , x);	\
  __OUTS1(s##c_p, x);

__OUTS(b, char);
__OUTS(w, short);
__OUTS(l, long);

#define outb(val, port)	\
  (__builtin_constant_p((port)) ? __outbc((val),(port)) : __outb((val),(port)))

#define outb_p(val, port)	\
  (__builtin_constant_p((port)) ? __outbc_p((val),(port)) : __outb_p((val),(port)))

#define outw(val, port)	\
  (__builtin_constant_p((port)) ? __outwc((val),(port)) : __outw((val),(port)))

#define outw_p(val, port)	\
  (__builtin_constant_p((port)) ? __outwc_p((val),(port)) : __outw_p((val),(port)))
  
#define outl(val, port)	\
  (__builtin_constant_p((port)) ? __outlc((val),(port)) : __outl((val),(port)))

#define outl_p(val, port)	\
  (__builtin_constant_p((port)) ? __outlc_p((val),(port)) : __outl_p((val),(port)))

#define inb(port)	\
  (__builtin_constant_p((port)) ? __inbc((port)) : __inb((port)))

#define inb_p(port)	\
  (__builtin_constant_p((port)) ? __inbc_p((port)) : __inb_p((port)))

#define inw(port)	\
  (__builtin_constant_p((port)) ? __inwc((port)) : __inw((port)))

#define inw_p(port)	\
  (__builtin_constant_p((port)) ? __inwc_p((port)) : __inw_p((port)))
  
#define inl(port)	\
  (__builtin_constant_p((port)) ? __inlc((port)) : __inl((port)))

#define inl_p(port)	\
  (__builtin_constant_p((port)) ? __inlc_p((port)) : __inl_p((port)))

#endif

