/*
 * linux/include/asm-arm/proc-arm2/segment.h
 *
 * Copyright (C) 1996 Russell King
 */

#ifndef __ASM_PROC_SEGMENT_H
#define __ASM_PROC_SEGMENT_H

static inline void __put_user(unsigned long x, void * y, int size)
{
	if (IS_USER_SEG) {
		switch (size) {
			case 1:
				__asm__(
				" strbt	%0, [%1]\n"
				: : "r" (x), "r" (y)
				: "lr", "cc");
				break;
			case 2:
				{ register unsigned long tmp;
				__asm__ __volatile__(
				" strbt %1, [%2], #1\n"
				" mov   %0, %1, lsr #8\n"
				" strbt %0, [%2]\n"
				: "=&r" (tmp)
				: "r" (x), "r" (y)
				: "2", "lr", "cc");
				}
				break;
			case 4:
				if ((int)y & 3) {
				register unsigned long tmp;
				__asm__ __volatile__(
				" strbt %1, [%2], #1\n"
				" mov   %0, %1, lsr #8\n"
				" strbt %0, [%2], #1\n"
				" mov   %0, %0, lsr #8\n"
				" strbt %0, [%2], #1\n"
				" mov   %0, %0, lsr #8\n"
				" strbt %0, [%2]\n"
				: "=&r" (tmp)
				: "r" (x), "r" (y)
				: "2", "lr", "cc");
				} else {
				__asm__(
				" strt  %0, [%1]\n"
				: : "r" (x), "r" (y)
				: "lr", "cc");
				}
				break;
			default:
				bad_user_access_length();
		}
	} else {
		switch (size) {
			case 1: *(unsigned char *)y = x; break;
			case 2: *(unsigned short *)y = x; break;
			case 4: *(unsigned long *)y = x; break;
			default:
				bad_user_access_length();
		}
	}
}

static inline unsigned long __get_user(const void *y, int size)
{
	unsigned long result1, result2;

	if (IS_USER_SEG) {
		switch (size) {
			case 1:
				__asm__(
				" ldrbt %0, [%1]\n"
				: "=r" (result1)
				: "r" (y)
				: "lr", "cc");
				return result1;
			case 2:
				__asm__(
				" ldrbt %0, [%2], #1\n"
				" ldrbt %1, [%2]\n"
				" orr   %0, %0, %1, lsl #8\n"
				: "=&r" (result1), "=r" (result2)
				: "r" (y)
				: "lr", "cc");
				return result1;
			case 4:
				if ((int)y & 3) {
				__asm__(
				" ldrt %0, [%2], #4\n"
				" ldrt %1, [%2]\n"
				: "=&r" (result1), "=r" (result2)
				: "r" ((int)y & ~3)
				: "lr", "cc");
				result1 >>= ((int)y & 3)*8;
				return result1 | (result2 << (32-((int)y & 3)*8));
				} else {
				__asm__(
				" ldrt %0, [%1]\n"
				: "=r" (result1)
				: "r" (y)
				: "lr", "cc");
				return result1;
				}
			default:
				return bad_user_access_length();
		}
	} else {
		switch (size) {
			case 1: return *(unsigned char *)y; break;
			case 2: return *(unsigned short *)y; break;
			case 4: return *(unsigned long *)y; break;
			default:
				return bad_user_access_length();
		}
	}
}		

#endif /* __ASM_PROC_SEGMENT_H */

