/*
 * linux/include/asm-arm/proc-arm2/system.h
 *
 * Copyright (C) 1996 Russell King
 */

#ifndef __ASM_PROC_SYSTEM_H
#define __ASM_PROC_SYSTEM_H

static inline unsigned long __xchg(unsigned long x, volatile void *ptr, int size)
{
	unsigned long temp0, temp1;
	switch (size) {
		case 1: __asm__ __volatile__(
			" mov   %0, pc\n"
			" orr   %1, %0, #0x08000000\n"
			" teqp  %1, #0\n"
			" mov   %1, %3\n"	/* Safe: in case it uses the same reg */
			" ldrb  %2, [%4]\n"
			" strb  %1, [%4]\n"
			" teqp  %0, #0\n"
			: "=r" (temp0), "=r" (temp1), "=r" (x)
			: "r" (x), "r" (ptr)
			: "memory");
			break;
		case 2: { unsigned long temp0, temp1;
			if (((long)ptr) & 1)
			{
				void *pc;
				__asm__("mov\t%0, pc": "=r" (pc));
				arm_malalignedptr("xchg", pc, ptr);
			}
			__asm__ __volatile__(
			" mov   %0, pc\n"
			" orr   %1, %0, #0x08000000\n"
			" teqp  %1, #0\n"
			" mov   %1, %3\n"
			" ldr	%2, [%4]\n"
			" strb  %1, [%4]\n"
			" mov   %1, %1, lsr #8\n"
			" strb  %1, [%4, #1]\n"
			" teqp  %0, #0\n"
			: "=r" (temp0), "=r" (temp1), "=r" (x)
			: "r" (x), "r" (ptr)
			: "memory");
			val &= 0xffff;
			break;
			}
		case 4: { unsigned long temp0, temp1;
			if (((long)ptr) & 3)
			{
				void *pc;
				__asm__("mov\t%0, pc": "=r" (pc));
				arm_malalignedptr("xchg", pc, ptr);
			}
			__asm__ __volatile__(
			" mov   %0, pc\n"
			" orr   %1, %0, #0x08000000\n"
			" teqp  %1, #0\n"
			" mov   %1, %3\n"
			" ldr   %2, [%4]\n"
			" str   %1, [%4]\n"
			" teqp  %0, #0\n"
			: "=r" (temp0), "=r" (temp1), "=r" (x)
			: "r" (x), "r" (ptr)
			: "memory");
			}
			break;
		default:
			{
				void *pc;
				__asm__("mov\t%0, pc": "=r" (pc));
				arm_invalidptr("xchg", pc, size);
				x = 0;
			}
	}
	return x;
}

/*
 * A couple of speedups for the ARM
 */
#define save_flags_cli(x)		\
	do {				\
	  unsigned long temp;		\
	  __asm__ __volatile__(		\
"	mov	%0, pc\n"		\
"	orr	%1, %0, #0x08000000\n"	\
"	and	%0, %0, #0x0c000000\n"	\
"	teqp	%1, #0\n"		\
	  : "=r" (x), "=r" (temp));	\
	} while (0)
	

#define sti()				\
	do {				\
	  unsigned long temp;		\
	  __asm__ __volatile__(		\
"	mov	%0, pc\n"		\
"	bic	%0, %0, #0x08000000\n"	\
"	teqp	%0, #0\n"		\
	  : "=r" (temp)			\
	  : );				\
	} while(0)

#define cli()				\
	do {				\
	  unsigned long temp;		\
	  __asm__ __volatile__(		\
"	mov	%0, pc\n"		\
"	orr	%0, %0, #0x08000000\n"	\
"	teqp	%0, #0\n"		\
	  : "=r" (temp)			\
	  : );				\
	} while(0)

#define save_flags(x)			\
	do {				\
	  __asm__ __volatile__(		\
"	mov	%0, pc\n"		\
"	and	%0, %0, #0x0c000000\n"	\
	  : "=r" (x) : );		\
	} while (0)

#define restore_flags(x)		\
	do {				\
	  unsigned long temp;		\
	  __asm__ __volatile__(		\
"	mov	%0, pc\n"		\
"	bic	%0, %0, #0x0c000000\n"	\
"	orr	%0, %0, %1\n"		\
"	teqp	%0, #0\n"		\
	  : "=r" (temp)			\
	  : "r" (x)			\
	  : "cc");			\
	} while (0)

#endif
