/*
 * include/asm-arm/processor.h
 *
 * Copyright (C) 1995 Russell King
 */

#ifndef __ASM_ARM_PROCESSOR_H
#define __ASM_ARM_PROCESSOR_H

/*
 * Bus types
 */
#define EISA_bus 0
#define EISA_bus__is_a_macro /* for versions in ksyms.c */
#define MCA_bus 0
#define MCA_bus__is_a_macro /* for versions in ksyms.c */

#define ARM_bus	1
#define ARM_bus__is_a_macro

/*
 * on arm2,3 wp does not work ok
 */
#define wp_works_ok 0
#define wp_works_ok__is_a_macro /* for versions in ksyms.c */

/*
 * User space process size: 3GB. This is hardcoded into a few places,
 * so don't change it unless you know what you are doing.
 */
#define TASK_SIZE       (0x01800000UL)

struct thread_struct {
  	unsigned long	uregs[5];		/* Only save user r0,r1,r2,r13,r14 */
  	unsigned long	sregs[11];		/* Only save svc r4-r14 */
  	unsigned long	fpe_save[140/4];	/* FPE save state */
  	unsigned long	address,		/* Address of fault */
  			trap_no,		/* Trap number */
  			error_code;		/* Error code of trap */
  	pgd_t		*memmap;		/* Memory map */
  	unsigned long	fs;			/* simulated fs */
};

#define INIT_MMAP { &init_mm, 0, 0x02000000, PAGE_SHARED, VM_READ | VM_WRITE | VM_EXEC }

#define INIT_TSS  { \
	{ 0, }, \
	{ 0, }, \
	{ 0, }, \
	0,0,0, \
	(long) &swapper_pg_dir, \
	1 \
}

static inline void start_thread(struct pt_regs * regs, unsigned long pc, unsigned long sp)
{
	int i;
	unsigned long *stack = (unsigned long *)sp;
	for(i=3; i <15; i++)	/* Initialise all registers to zero */
	  regs->uregs[i] = 0;
	regs->ARM_pc = pc;		/* pc */
	regs->ARM_sp = sp;		/* sp */
	regs->ARM_r2 = stack[2];	/* r2 (envp) */
	regs->ARM_r1 = stack[1];	/* r1 (argv) */
	regs->ARM_r0 = stack[0];	/* r0 (argc) */
}

/*
 * Return a saved PC of a blocked thread
 */
extern inline unsigned long thread_saved_pc(struct thread_struct *t)
{
	return ((unsigned long *)t->sregs[9])[2];
}
#endif /* __ASM_ARM_PROCESSOR_H */

